#!/bin/bash
#shell script for handling ACE real-time data and GFZ Potsdam Kp/ap quicklook data
#duration: ~8 seconds
date --utc -d 'today' '+%Y-%m-%d %H:%M:%S ace_realtime_ap_plot.sh'
thisFile=$(readlink -f $BASH_SOURCE)
thisFilePath=$(dirname $(readlink -f $BASH_SOURCE))

#dates:
today=`date --utc -d 'today' '+%Y%m%d'`
day1ago=`date --utc -d '1 day ago' '+%Y%m%d'`
day2ago=`date --utc -d '2 days ago' '+%Y%m%d'`
day3ago=`date --utc -d '3 days ago' '+%Y%m%d'`


########## ACE rt data ##################

#load latest 2 hours and last 4 days::
swpcAceUrl="http://www.swpc.noaa.gov/ftpdir/lists/ace/"
magFile="ace_mag_1m.txt"
swepamFile="ace_swepam_1m.txt"
wget $swpcAceUrl$magFile -q -N
wget $swpcAceUrl$swepamFile -q -N
wget $swpcAceUrl$today"_"$magFile -q -N
wget $swpcAceUrl$today"_"$swepamFile -q -N
wget $swpcAceUrl$day1ago"_"$magFile -q -N
wget $swpcAceUrl$day1ago"_"$swepamFile -q -N
wget $swpcAceUrl$day2ago"_"$magFile -q -N
wget $swpcAceUrl$day2ago"_"$swepamFile -q -N
wget $swpcAceUrl$day3ago"_"$magFile -q -N
wget $swpcAceUrl$day3ago"_"$swepamFile -q -N

#create combined mag+swepam data files:
magFileTemp="ace_mag_1m_temp.txt"
swepamFileTemp="ace_swepam_1m_temp.txt"
awk '!/#/ && !/:/ {print}' $magFile > $magFileTemp
awk '!/#/ && !/:/ {print}' $swepamFile > $swepamFileTemp
paste $magFileTemp $swepamFileTemp > ace_magswepam_1m_last2hours.txt	#!!mag data may be shifted to swepam data by 1 minute!!
awk '!/#/ && !/:/ {print}' $today"_"$magFile > $today"_"$magFileTemp
awk '!/#/ && !/:/ {print}' $today"_"$swepamFile > $today"_"$swepamFileTemp
paste $today"_"$magFileTemp $today"_"$swepamFileTemp > $today"_ace_magswepam_1m_temp.txt"
awk '!/#/ && !/:/ {print}' $day1ago"_"$magFile > $day1ago"_"$magFileTemp
awk '!/#/ && !/:/ {print}' $day1ago"_"$swepamFile > $day1ago"_"$swepamFileTemp
paste $day1ago"_"$magFileTemp $day1ago"_"$swepamFileTemp > $day1ago"_ace_magswepam_1m_temp.txt"
awk '!/#/ && !/:/ {print}' $day2ago"_"$magFile > $day2ago"_"$magFileTemp
awk '!/#/ && !/:/ {print}' $day2ago"_"$swepamFile > $day2ago"_"$swepamFileTemp
paste $day2ago"_"$magFileTemp $day2ago"_"$swepamFileTemp > $day2ago"_ace_magswepam_1m_temp.txt"
awk '!/#/ && !/:/ {print}' $day3ago"_"$magFile > $day3ago"_"$magFileTemp
awk '!/#/ && !/:/ {print}' $day3ago"_"$swepamFile > $day3ago"_"$swepamFileTemp
paste $day3ago"_"$magFileTemp $day3ago"_"$swepamFileTemp > $day3ago"_ace_magswepam_1m_temp.txt"

#combine all daily files:
last4daysFile="ace_magswepam_1m_last4days.txt"
paste -s -d "\n" $day3ago"_ace_magswepam_1m_temp.txt" $day2ago"_ace_magswepam_1m_temp.txt" $day1ago"_ace_magswepam_1m_temp.txt" $today"_ace_magswepam_1m_temp.txt" > $last4daysFile

#delete temporary files:
rm *_1m_temp.txt -f
rm *_1m.txt -f

########### GFZ ap data ##########################

#load latest Kp/ap file:
gfzKpUrl="http://www-app3.gfz-potsdam.de/kp_index/"
kpFile="qlyymm.wdc"
kpFileTemp="qlyymm_temp.wdc"
wget $gfzKpUrl$kpFile -q -N
awk 'BEGIN{FIELDWIDTHS="2 2 2 4 2 2 2 2 2 2 2 2 2 3 3 3 3 3 3 3 3 3 7"} {print "20"$1" "$2" "$3" 0300 "$6" "$15; print "20"$1" "$2" "$3" 0600 "$7" "$16; print "20"$1" "$2" "$3" 0900 "$8" "$17; print "20"$1" "$2" "$3" 1200 "$9" "$18; print "20"$1" "$2" "$3" 1500 "$10" "$19; print "20"$1" "$2" "$3" 1800 "$11" "$20; print "20"$1" "$2" "$3" 2100 "$12" "$21; print "20"$1" "$2" "$3" 2400 "$13" "$22}' $kpFile > $kpFileTemp


########### convert VBz to ap nowcast ################

#create data file (convert minutely to hourly data):
last4days1hFile="ace_magswepam_1h_last4days.txt"
awk '{if ($10 != -999.9) {x=x+$10; xc++}; if ($22 != -9999.9) {y=y+$22; yc++}; if (($6 % 3600 == 0 && $6 != 0 || $6 == 86340) && (xc > 0 && yc > 0)) {print $1" "$2" "$3" "$4" "x/xc" "y/yc; x=0; xc=0; y=0; yc=0} else if (($6 % 3600 == 0 && $6 != 0 || $6 == 86340) && (xc == 0 && yc != 0)) {print $1" "$2" "$3" "$4" 0.0 "y/yc; x=0; xc=0; y=0; yc=0} else if (($6 % 3600 == 0 && $6 != 0 || $6 == 86340) && (xc > 0 && yc > 0)) {print $1" "$2" "$3" "$4" "x/xc" 0.0"; x=0; xc=0; y=0; yc=0} else if (($6 % 3600 == 0 && $6 != 0 || $6 == 86340) && (xc == 0 && yc == 0)) {print $1" "$2" "$3" "$4" 0.0 0.0"; x=0; xc=0; y=0; yc=0}}' $last4daysFile > $last4days1hFile


#create data file (convert hourly to 3-hourly min(v*Bz) data):
last4days3hmaxFile="ace_magswepam_3hmax_last4days.txt"
awk 'BEGIN{count=0; Bzmin3h=9999999} {count++; if (Bzmin3h > ($5*$6)) {Bzmin3h=($5*$6)}; if (count%3==0) {print $1, $2, $3, $4, Bzmin3h; Bzmin3h=9999999}}' $last4days1hFile > $last4days3hmaxFile

#convert 3hrmin(v*Bz) data to ap via relational table:
conversionTableFile="ConversionTablevBzToapB.txt"
apEstimateFile="ap_estimate_last4days.txt"
cfile=( $( < $conversionTableFile ) ); columns=15; k=0; for i in ${cfile[*]}; do	if [ $(($k % $columns)) -eq 0 ]; then counts[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 1 ]; then xBinPos[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 2 ]; then xBin[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 5 ]; then apMixed[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 9 ]; then ap1sigmaMinMix[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 10 ]; then ap1sigmaMaxMix[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 11 ]; then ap2sigmaMinMix[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 12 ]; then ap2sigmaMaxMix[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 13 ]; then ap3sigmaMinMix[$(($k / $columns))]=$i; fi;	if [ $(($k % $columns)) -eq 14 ]; then ap3sigmaMaxMix[$(($k / $columns))]=$i; fi;	k=$(($k+1)); done; awk -v xBinAwk="${xBin[*]}" -v apMixedAwk="${apMixed[*]}" -v arrayLength="${#apMixed[*]}" -v ap1sigmaMinMixAwk="${ap1sigmaMinMix[*]}" -v ap1sigmaMaxMixAwk="${ap1sigmaMaxMix[*]}" -v ap2sigmaMinMixAwk="${ap2sigmaMinMix[*]}" -v ap2sigmaMaxMixAwk="${ap2sigmaMaxMix[*]}" -v ap3sigmaMinMixAwk="${ap3sigmaMinMix[*]}" -v ap3sigmaMaxMixAwk="${ap3sigmaMaxMix[*]}" 'BEGIN{split(apMixedAwk, a, / /); split(xBinAwk, b, / /); split(ap1sigmaMinMixAwk, ap1min, / /); split(ap1sigmaMaxMixAwk, ap1max, / /); split(ap2sigmaMinMixAwk, ap2min, / /); split(ap2sigmaMaxMixAwk, ap2max, / /); split(ap3sigmaMinMixAwk, ap3min, / /); split(ap3sigmaMaxMixAwk, ap3max, / /);} {vBz=$5; for (i=1; i<arrayLength; i++) {if (b[i]<=vBz && vBz < b[i+1]) print $1,$2,$3,$4,$5,a[i],ap1min[i],ap1max[i],ap2min[i],ap2max[i],ap3min[i],ap3max[i]}; if (b[arrayLength]<=vBz) {print $1,$2,$3,$4,$5,a[arrayLength],ap1min[arrayLength],ap1max[arrayLength],ap2min[arrayLength],ap2max[arrayLength],ap3min[arrayLength],ap3max[arrayLength]}; if (vBz<b[1]) {print $1,$2,$3,$4,$5,a[1],ap1min[1],ap1max[1],ap2min[1],ap2max[1],ap3min[1],ap3max[1]}} END{}' $last4days3hmaxFile > $apEstimateFile

########### CH forecast #######################

#create CH SW forecast data folder for today and yesterday:
ugrazCHforecastUrl="http://www.uni-graz.at/igam-sophy/comesep/products/"
#echo "forecast_`date --utc -d 'now' '+%Y%m%dT%H00'`.txt"
CH_directory="CH_forecast_data"
wget --recursive --level=1 --no-parent --no-directories --directory-prefix="$CH_directory" -A "forecast_`date --utc -d 'today' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d 'yesterday' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '2 days ago' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '3 days ago' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '4 days ago' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '5 days ago' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '6 days ago' '+%Y%m%d'`T*00.txt","forecast_`date --utc -d '7 days ago' '+%Y%m%d'`T*00.txt" $ugrazCHforecastUrl -q -N
#delete robots.txt which is also downloaded for a reason I don't know:
rm "$CH_directory/robots.txt" -f

#convert file to single line:
outputFile="$CH_directory/CH_forecast_merged.txt"
outputFile2="$CH_directory/CH_forecast_merged2.txt"
outputFile3="$CH_directory/CH_forecast_merged2_with_ap-forecast.txt"

#datum="22-Aug-2013 14:00:06.840"
#echo "convert date: `date --utc -d "$datum" '+%Y %m %d  %H%M'`"
#dateYYYY="2013"
#dateDOY="241.24"
#dateSOY=$(echo "$dateDOY*24*60*60" | bc)
#echo $dateSOY
#echo "convert date1,5: `date --utc -d "$dateYYYY-01-01 -1 day +$dateSOY seconds" '+%Y %m %d  %H%M'`"
#echo "convert date1,7: `date --utc -d "$dateYYYY-01-01 -1 day +$(echo "$dateDOY*24*60*60" | bc) seconds" '+%Y %m %d  %H%M'`"

#echo $dateDOY
#datedd=$(echo "$dateDOY/1" | bc)
#echo $datedd
#bc <<< "(($dateDOY%1)*24)"
#dateHH=$(bc <<<     "(($dateDOY%1)*24)/1")
#echo $dateHH
#bc <<< "(((($dateDOY%1)*24)%1)*60)"
#dateMM=$(bc <<<   "(((($dateDOY%1)*24)%1)*60)/1")
#echo $dateMM
#bc <<< "(((((($dateDOY%1)*24)%1)*60)%1)*60)"
#dateSS=$(bc <<< "(((((($dateDOY%1)*24)%1)*60)%1)*60)/1")
#echo $dateSS
#echo "convert date2: `date --utc -d "$dateYYYY-01-01 -1 day +$datedd days +$dateHH hours +$dateMM minutes" '+%Y %m %d  %H%M'`"

#print first line (e.g. column header):
echo "date       time        `awk 'NR%2==0 {print $0} END {}' $CH_directory/forecast_*00.txt | awk 'BEGIN{RS=""; FS="\n"} {print $1, $2} END {}'`" > $outputFile
for gfile in $CH_directory/forecast_*00.txt; do
	awk 'NR%2==1 {print $0} END {}' $gfile | awk 'BEGIN{RS=""; FS="\n"} {print $1, $2, $3, $4} END {}' >> $outputFile
done
#delete files:
rm $CH_directory/forecast_*00.txt -f

#convert date format (Aug to 08):
#awk 'NR!=1 {cmd="date --utc --date=\""$1" "$2"\" +\"%Y %m %d %H%M \""; cmd | getline convdate; print convdate $0}' $outputFile > $outputFile3
#convert date format (DOY to date):
#awk 'NR!=1 {cmd="date --utc --date=\""$3"-01-01 -1 day +$(echo \""$4"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd | getline convdate; print convdate $0}' $outputFile > $outputFile2
awk 'NR!=1 {cmd="date --utc --date=\""$3"-01-01 -1 day +$(echo \""$4"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd | getline convdate; close(cmd); cmd_v="date --utc --date=\""$6"-01-01 -1 day +$(echo \""$7"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd_v | getline convdate_v; close(cmd_v); cmd_T="date --utc --date=\""$9"-01-01 -1 day +$(echo \""$10"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd_T | getline convdate_T; close(cmd_T); cmd_n="date --utc --date=\""$12"-01-01 -1 day +$(echo \""$13"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd_n | getline convdate_n; close(cmd_n); cmd_B="date --utc --date=\""$15"-01-01 -1 day +$(echo \""$16"*24*60*60\" | bc) seconds\" +\"%Y %m %d %H%M \""; cmd_B | getline convdate_B; close(cmd_B); print convdate convdate_v convdate_T convdate_n convdate_B $0}' $outputFile > $outputFile2

#calculate ap from CH SW velocity forecast:
#CH_directory="CH_forecast_data"
#outputFile2="$CH_directory/CH_forecast_merged2.txt"
#outputFile3="$CH_directory/CH_forecast_merged2_with_ap_forecast.txt"
conversionTableFileVToap="ConversionTableVToapB.txt"
{
cfile=( $( < $conversionTableFileVToap ) ); 
columns=15; 
k=0; 
for i in ${cfile[*]}; 
do
	if [ $(($k % $columns)) -eq 0 ]; then counts[$(($k / $columns))]=$i; fi;
	if [ $(($k % $columns)) -eq 1 ]; then xBinPos[$(($k / $columns))]=$i; fi;
	if [ $(($k % $columns)) -eq 2 ]; then xBin[$(($k / $columns))]=$i; fi;
	if [ $(($k % $columns)) -eq 5 ]; then apMixed[$(($k / $columns))]=$i; fi;
	k=$(($k+1)); 
done; 
awk -v xBinAwk="${xBin[*]}" -v apMixedAwk="${apMixed[*]}" -v arrayLength="${#apMixed[*]}" 'BEGIN{split(apMixedAwk, a, / /); split(xBinAwk, b, / /);} {V=$28; for (i=1; i<arrayLength; i++) {if (b[i]<=V && V < b[i+1]) print $0, a[i]}; if (b[arrayLength]<=V) {print $0, a[arrayLength]}; if (V<b[1]) {print $0, a[1]}}' $outputFile2 > $outputFile3
}

########## GFT ap data ##########################

#get *.asc file name from "farm_nrt.cfg", read GFT *.asc file and write conversion into "gft_ap_data.txt":
gft_path="/home/raid0/mvenzmer/rssfeeds/sri_gft_ap"
fileName="`awk '/FNDAT/ && !/FORECAST/ {print substr($0,8,24)} END{}' $gft_path"/farm_nrt.cfg"`"
echo $fileName
#echo ${fileName:7:24}
#var=${fileName#"FNDAT='"}
#echo ${var%"'"}
awk 'NR!=1 { printf ("%04d %3d %4d %02d %02d %02d00 %3d %3d %3d\n", $1,$2,$3,$4,$5,$7,$6,$8,$9)} END{}' "$gft_path/$fileName" > "gft_ap/gft_ap_data.txt"

fileNameTime="`awk '/FNDAT/ && !/FORECAST/ {print substr($0,14,14)} END{}' $gft_path"/farm_nrt.cfg"`"
echo $fileNameTime

find $gft_path -name 'ace-mag-rt-*.asc' ! -name "ace-mag-rt-$fileNameTime.asc" -type f | xargs rm -f
find $gft_path -name 'ace-swepam-rt-*.asc' ! -name "ace-swepam-rt-$fileNameTime.asc" -type f | xargs rm -f
find $gft_path -name 'kp-rt-*.asc' ! -name "kp-rt-$fileNameTime.asc" -type f | xargs rm -f
find $gft_path -name 'dst-rt-*.asc' ! -name "dst-rt-$fileNameTime.asc" -type f | xargs rm -f
find $gft_path -name 'ap-fc-*.xml' ! -name "ap-fc-$fileNameTime.xml" -type f | xargs rm -f
find $gft_path -name 'ap-fc-*.asc' ! -name "ap-fc-$fileNameTime.asc" -type f | xargs rm -f
#rm $gft_path/ace-mag-rt-*.asc -f
#rm $gft_path/ace-swepam-rt-*.asc -f
#rm $gft_path/kp-rt-*.asc -f
#rm $gft_path/dst-rt-*.asc -f
#rm $gft_path/ap-fc-*.asc -f
#rm $gft_path/ap-fc-*.xml -f

########## Gnuplot ########################

#run gnuplot: 
gnuplot ace_realtime_ap_CH_GFT_gnuplot_script.txt

echo "`date --utc -d 'now' '+%Y-%m-%d %H:%M:%S %Z on %A '`$thisFile" >> $thisFilePath"/log.txt"

